package com.xjrsoft.module.evaluate.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.bean.BeanUtil;
import com.xjrsoft.common.model.result.RT;
import com.xjrsoft.module.evaluate.dto.AddEvaluateItemDto;
import com.xjrsoft.module.evaluate.dto.MakeEvaluateItemDto;
import com.xjrsoft.module.evaluate.entity.EvaluateItem;
import com.xjrsoft.module.evaluate.service.IEvaluateItemService;
import com.xjrsoft.module.evaluate.vo.EvaluateItemVo;
import com.xjrsoft.module.evaluate.vo.MakeEvaluateItemVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
* @title: 评价项
* @Author szs
* @Date: 2024-01-15
* @Version 1.0
*/
@RestController
@RequestMapping("/evaluate" + "/evaluateItem")
@Api(value = "/evaluate"  + "/evaluateItem",tags = "评价项代码")
@AllArgsConstructor
public class EvaluateItemController {


    private final IEvaluateItemService evaluateItemService;

    @GetMapping(value = "/evaluate-item")
    @ApiOperation(value="评价模板维护评价项页面")
    @SaCheckPermission("evaluateitem:detail")
    public RT<MakeEvaluateItemVo> list(@Valid MakeEvaluateItemDto dto){
        MakeEvaluateItemVo makeEvaluateItemVo = evaluateItemService.getMakeEvaluateItem(dto);
        return RT.ok(makeEvaluateItemVo);
    }

    @GetMapping(value = "/info")
    @ApiOperation(value="根据id查询评价项信息")
    @SaCheckPermission("evaluateitem:detail")
    public RT<EvaluateItemVo> info(@RequestParam Long id){
        EvaluateItem evaluateItem = evaluateItemService.getById(id);
        if (evaluateItem == null) {
           return RT.error("找不到此数据！");
        }
        return RT.ok(BeanUtil.toBean(evaluateItem, EvaluateItemVo.class));
    }


    @PostMapping
    @ApiOperation(value = "新增评价项")
    @SaCheckPermission("evaluateitem:add")
    public RT<Boolean> add(@Valid @RequestBody List<AddEvaluateItemDto> dto){
        boolean isSuccess = evaluateItemService.addEvaluateItem(dto);
        return RT.ok(isSuccess);
    }
}