package com.xjrsoft.module.textbook.entity;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableName;
import com.github.yulichang.annotation.EntityMapping;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;


/**
* @title: 教材管理
* @Author szs
* @Date: 2023-12-25
* @Version 1.0
*/
@Data
@TableName("textbook")
@ApiModel(value = "textbook", description = "教材管理")
public class Textbook implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
    * 主键编号
    */
    @ApiModelProperty("主键编号")
    @TableId
    private Long id;
    /**
    * 创建人
    */
    @ApiModelProperty("创建人")
    @TableField(fill = FieldFill.INSERT)
    private Long createUserId;
    /**
    * 创建时间
    */
    @ApiModelProperty("创建时间")
    @TableField(fill = FieldFill.INSERT)
    private Date createDate;
    /**
    * 修改人
    */
    @ApiModelProperty("修改人")
    @TableField(fill = FieldFill.UPDATE)
    private Long modifyUserId;
    /**
    * 修改时间
    */
    @ApiModelProperty("修改时间")
    @TableField(fill = FieldFill.UPDATE)
    private Date modifyDate;
    /**
    * 删除标记
    */
    @ApiModelProperty("删除标记")
    @TableField(fill = FieldFill.INSERT)
    @TableLogic
    private Integer deleteMark;
    /**
    * 有效标志
    */
    @ApiModelProperty("有效标志")
    @TableField(fill = FieldFill.INSERT)
    private Integer enabledMark;
    /**
    * 序号
    */
    @ApiModelProperty("序号")
    private Integer sortCode;
    /**
    * 国际标准刊号
    */
    @ApiModelProperty("国际标准刊号")
    private String issn;
    /**
    * 国际标准书号
    */
    @ApiModelProperty("国际标准书号")
    private String isbn;
    /**
    * 书名
    */
    @ApiModelProperty("书名")
    private String bookName;
    /**
    * 出版社
    */
    @ApiModelProperty("出版社")
    private String publishingHouse;
    /**
    * 主编
    */
    @ApiModelProperty("主编")
    private String editorInChief;
    /**
     * 教材核心信息管理id(textbook_core_attribute)
     */
    @ApiModelProperty("教材核心信息管理id(textbook_core_attribute)")
    private Long textbookCoreAttributeId;
    /**
    * 学科组管理编号(subject_group)
    */
    @ApiModelProperty("学科组管理编号(subject_group)")
    private Long subjectGroupId;
    /**
    * 学期ID(base_semester)
    */
    @ApiModelProperty("学期ID(base_semester)")
    private Long baseSemesterId;
    /**
    * 课程编号(base_course_subject)
    */
    @ApiModelProperty("课程编号(base_course_subject)")
    private Long courseSubjectId;
    /**
     * 使用年级(base_grade)
     */
    @ApiModelProperty("使用年级(base_grade)")
    private Long gradeId;
    /**
    * 版本
    */
    @ApiModelProperty("版本")
    private String version;
    /**
    * 是否为规划教材(xjr_dictionary_item[judgment_method_1])
    */
    @ApiModelProperty("是否为规划教材(xjr_dictionary_item[judgment_method_1])")
    private String isTextbookPlan;
    /**
    * 教材类型(xjr_dictionary_item[textbook_type])
    */
    @ApiModelProperty("教材类型(xjr_dictionary_item[textbook_type])")
    private String textbookType;
    /**
    * 规格型号
    */
    @ApiModelProperty("规格型号")
    private String specificationsModels;
    /**
    * 估价(元)
    */
    @ApiModelProperty("估价(元)")
    private BigDecimal appraisalPrice;
    /**
    * 定价(元)
    */
    @ApiModelProperty("定价(元)")
    private BigDecimal price;
    /**
    * 折扣
    */
    @ApiModelProperty("折扣")
    private Double discount;
    /**
    * 小计（元）
    */
    @ApiModelProperty("小计（元）")
    private BigDecimal subtotal;
    /**
    * 库存
    */
    @ApiModelProperty("库存")
    private Integer stock;

    /**
    * textbookClassRelation
    */
    @ApiModelProperty("textbookClassRelation子表")
    @TableField(exist = false)
    @EntityMapping(thisField = "id", joinField = "textbookId")
    private List<TextbookClassRelation> textbookClassRelationList;
    /**
    * textbookSubscriptionRecord
    */
    @ApiModelProperty("textbookSubscriptionRecord子表")
    @TableField(exist = false)
    @EntityMapping(thisField = "id", joinField = "wfTextbookSubscriptionId")
    private List<TextbookSubscriptionRecord> textbookSubscriptionRecordList;

    /**
     * 使用类型(单位：学期)
     */
    @ApiModelProperty("使用类型(单位：学期)")
    private Integer useType;


    @ApiModelProperty("是否校企合作开发教材")
    private Integer isSecd;

    @ApiModelProperty("出版日期")
    private Date publishingDate;

    @ApiModelProperty("分类号")
    private String category;

    @ApiModelProperty("规划批次")
    private String planBatch;

    @ApiModelProperty("编著作总数")
    private Integer workTotalCount;

    @ApiModelProperty("教材类型")
    private String textbookCategory;
}