
-- ------------------------------------------------------------------奖学金管理--------------------------------------------------------------------
-- ----------------------------
-- 奖学金类别
-- ----------------------------
DROP TABLE IF EXISTS base_student_scholarship_category;
CREATE TABLE base_student_scholarship_category
(
    id BIGINT NOT NULL COMMENT '主键编号',
    `create_user_id` BIGINT NULL DEFAULT NULL COMMENT '创建人',
    `create_date` datetime NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id` BIGINT NULL DEFAULT NULL COMMENT '修改人',
    `modify_date` datetime NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark` INT NOT NULL COMMENT '删除标记',
    `enabled_mark` INT NOT NULL COMMENT '有效标志',
    `sort_code` INT NULL DEFAULT NULL COMMENT '序号',
    `name` VARCHAR(200) NOT NULL COMMENT '名称',
    `scholarship_source` varchar(20) NOT NULL DEFAULT 0 COMMENT '奖学金来源(xjr_dictionary_item[scholarship_source])',
    `base_semester_id` bigint NULL DEFAULT NULL COMMENT '学期ID(base_semester)',
    `total_amount` double NOT NULL DEFAULT 0 COMMENT '总金额',
    `scholarship_level` int NOT NULL DEFAULT 0 COMMENT '奖学金等级 0=无等级',
    `remark` VARCHAR(1000) COMMENT '备注',
    PRIMARY KEY (`id`)
) ENGINE=INNODB DEFAULT CHARSET=utf8mb4 COLLATE = utf8mb4_0900_ai_ci COMMENT '奖学金类别';

-- ----------------------------
-- 奖学金级别
-- ----------------------------
DROP TABLE IF EXISTS base_student_scholarship_level;
CREATE TABLE base_student_scholarship_level
(
    id BIGINT NOT NULL COMMENT '主键编号',
    `create_user_id` BIGINT NULL DEFAULT NULL COMMENT '创建人',
    `create_date` datetime NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id` BIGINT NULL DEFAULT NULL COMMENT '修改人',
    `modify_date` datetime NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark` INT NOT NULL COMMENT '删除标记',
    `enabled_mark` INT NOT NULL COMMENT '有效标志',
    `sort_code` INT NULL DEFAULT NULL COMMENT '序号',
    `base_student_scholarship_category_id` bigint NULL DEFAULT NULL COMMENT '奖学金类别ID(base_student_scholarship_category)',
    `level` int NOT NULL DEFAULT 0 COMMENT '奖学金等级 0=无等级',
    `amount` double NOT NULL DEFAULT 0 COMMENT '金额',
    PRIMARY KEY (`id`)
) ENGINE=INNODB DEFAULT CHARSET=utf8mb4 COLLATE = utf8mb4_0900_ai_ci COMMENT '奖学金级别';

-- ----------------------------
-- 奖学金申请
-- ----------------------------
DROP TABLE IF EXISTS base_student_scholarship_applicant;
CREATE TABLE base_student_scholarship_applicant
(
    id BIGINT NOT NULL COMMENT '主键编号',
    `create_user_id` BIGINT NULL DEFAULT NULL COMMENT '创建人',
    `create_date` datetime NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id` BIGINT NULL DEFAULT NULL COMMENT '修改人',
    `modify_date` datetime NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark` INT NOT NULL COMMENT '删除标记',
    `enabled_mark` INT NOT NULL COMMENT '有效标志',
    `sort_code` INT NULL DEFAULT NULL COMMENT '序号',
    `base_student_scholarship_category_id` bigint NULL DEFAULT NULL COMMENT '奖学金类别ID(base_student_scholarship_category)',
    `scholarship_level` int NOT NULL DEFAULT 0 COMMENT '奖学金等级 0=无等级',
    `amount` double NOT NULL DEFAULT 0 COMMENT '金额',
    `applicant_user_id` BIGINT NULL DEFAULT NULL COMMENT '申请人',
    `base_semester_id` bigint NULL DEFAULT NULL COMMENT '学期ID(base_semester)',
    `grade_name` VARCHAR(200) NULL DEFAULT NULL COMMENT '年级名称',
    `class_name` VARCHAR(200) NULL DEFAULT NULL COMMENT '班级名称',
    `name` VARCHAR(200) NULL DEFAULT NULL COMMENT '姓名',
    `student_id` VARCHAR(200) NULL DEFAULT NULL COMMENT '学号',
    `gender_name` VARCHAR(200) NULL DEFAULT NULL COMMENT '性别',
    `enroll_type_cn` VARCHAR(200) NULL DEFAULT NULL COMMENT '招生类型名称',
    `major_name` VARCHAR(200) NULL DEFAULT NULL COMMENT '在读专业',
    `award_date` date NULL DEFAULT NULL COMMENT '获奖日期',
    `bank_type` VARCHAR(20) NULL DEFAULT NULL COMMENT '收款银行(xjr_dictionary_item[bank_type])',
    `bank_no` VARCHAR(200) NULL DEFAULT NULL COMMENT '银行卡号',
    `file_id` bigint NULL DEFAULT NULL COMMENT '文件ID(xjr_file)',
    `status` INT NOT NULL default 0 COMMENT '状态（1:结束 0:未结束）',
    `release_status` INT NOT NULL default 0 COMMENT '发放状态（1:已发放 0:未发放）',
    PRIMARY KEY (`id`)
) ENGINE=INNODB DEFAULT CHARSET=utf8mb4 COLLATE = utf8mb4_0900_ai_ci COMMENT '奖学金申请';
-- ------------------------------------------------------------------奖学金管理--------------------------------------------------------------------

-- ------------------------------------------------------------------用户--------------------------------------------------------------------
ALTER TABLE xjr_user ADD COLUMN `is_change_password` INT NOT NULL default 0 DEFAULT 1 COMMENT '是否需要修改密码（1:需要 0:不需要）' AFTER credential_type;
-- ------------------------------------------------------------------用户--------------------------------------------------------------------

-- ------------------------------------------------------------------助学金管理--------------------------------------------------------------------
-- ----------------------------
-- 助学金项目
-- ----------------------------
DROP TABLE IF EXISTS base_student_bursaries_project;
CREATE TABLE base_student_bursaries_project
(
    id BIGINT NOT NULL COMMENT '主键编号',
    `create_user_id` BIGINT NULL DEFAULT NULL COMMENT '创建人',
    `create_date` datetime NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id` BIGINT NULL DEFAULT NULL COMMENT '修改人',
    `modify_date` datetime NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark` INT NOT NULL COMMENT '删除标记',
    `enabled_mark` INT NOT NULL COMMENT '有效标志',
    `sort_code` INT NULL DEFAULT NULL COMMENT '序号',
    `name` VARCHAR(200) NOT NULL COMMENT '名称',
    `bursaries_type` varchar(20) NOT NULL DEFAULT 0 COMMENT '助学金类型(xjr_dictionary_item[bursaries_type])',
    `base_semester_id` bigint NULL DEFAULT NULL COMMENT '学期ID(base_semester)',
    `total_amount` double NOT NULL DEFAULT 0 COMMENT '总金额',
    `start_time` date NULL DEFAULT NULL COMMENT '开始时间',
    `end_time` date NULL DEFAULT NULL COMMENT '结束时间',
    `remark` VARCHAR(1000) NULL DEFAULT NULL COMMENT '备注',
    PRIMARY KEY (`id`)
) ENGINE=INNODB DEFAULT CHARSET=utf8mb4 COLLATE = utf8mb4_0900_ai_ci COMMENT '助学金项目';

-- ----------------------------
-- 助学金申请
-- ----------------------------
DROP TABLE IF EXISTS base_student_bursaries_applicant;
CREATE TABLE base_student_bursaries_applicant
(
    id BIGINT NOT NULL COMMENT '主键编号',
    `create_user_id` bigint NULL DEFAULT NULL COMMENT '创建人',
    `create_date` datetime NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id` bigint NULL DEFAULT NULL COMMENT '修改人',
    `modify_date` datetime NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark` INT NOT NULL COMMENT '删除标记',
    `enabled_mark` INT NOT NULL COMMENT '有效标志',
    `sort_code` INT NULL DEFAULT NULL COMMENT '序号',
    `base_student_bursaries_project_id` bigint NULL DEFAULT NULL COMMENT '助学金项目ID(base_student_bursaries_project)',
    `applicant_user_id` bigint NULL DEFAULT NULL COMMENT '申请人',
    `base_semester_id` bigint NULL DEFAULT NULL COMMENT '学期ID(base_semester)',
    `class_id` bigint NULL DEFAULT NULL COMMENT '班级编号',
    `file_id` bigint NULL DEFAULT NULL COMMENT '文件ID(xjr_file)',
    `status` INT NOT NULL default 0 COMMENT '状态（1:结束 0:未结束）',
    PRIMARY KEY (`id`)
) ENGINE=INNODB DEFAULT CHARSET=utf8mb4 COLLATE = utf8mb4_0900_ai_ci COMMENT '助学金申请';

-- ----------------------------
-- 助学金申请学生信息
-- ----------------------------
DROP TABLE IF EXISTS base_student_bursaries_student;
CREATE TABLE base_student_bursaries_student
(
    id BIGINT NOT NULL COMMENT '主键编号',
    `base_student_bursaries_applicant_id` bigint NULL DEFAULT NULL COMMENT '助学金申请ID(base_student_bursaries_applicant)',
    `user_id`  BIGINT NULL DEFAULT NULL COMMENT'学生id',
    `grade_name` VARCHAR(200) NULL DEFAULT NULL COMMENT '年级名称',
    `class_name` VARCHAR(200) NULL DEFAULT NULL COMMENT '班级名称',
    `name` VARCHAR(200) NULL DEFAULT NULL COMMENT '姓名',
    `student_id` VARCHAR(200) NULL DEFAULT NULL COMMENT '学号',
    `gender_name` VARCHAR(200) NULL DEFAULT NULL COMMENT '性别',
    `enroll_type_cn` VARCHAR(200) NULL DEFAULT NULL COMMENT '招生类型名称',
    `major_name` VARCHAR(200) NULL DEFAULT NULL COMMENT '在读专业',
    `bank_type` VARCHAR(20) NULL DEFAULT NULL COMMENT '收款银行(xjr_dictionary_item[bank_type])',
    `bank_no` VARCHAR(200) NULL DEFAULT NULL COMMENT '银行卡号',
    `applicant_amount` double NOT NULL DEFAULT 0 COMMENT '申请金额',
    PRIMARY KEY (`id`)
) ENGINE=INNODB DEFAULT CHARSET=utf8mb4 COLLATE = utf8mb4_0900_ai_ci COMMENT '助学金申请学生信息';
-- ------------------------------------------------------------------助学金管理--------------------------------------------------------------------








