-- 思政德育活动
DROP TABLE IF EXISTS `moral_event`;
CREATE TABLE moral_event
(
    id                 bigint       not null comment '主键编号'
        primary key,
    create_user_id     bigint       null comment '创建人',
    create_date        datetime     null comment '创建时间',
    modify_user_id     bigint       null comment '修改人',
    modify_date        datetime     null comment '修改时间',
    delete_mark        int          not null comment '删除标记',
    enabled_mark       int          not null comment '有效标志',
    sort_code          int          null comment '序号',

    moral_event_name   varchar(255) not null comment '德育活动名称',
    belong_theme       varchar(255) null comment '所属主题',
    event_plate        varchar(255) null comment '活动板块',
    event_type         varchar(255) not null comment '活动类型',
    event_theme        varchar(255) null comment '活动主题',
    event_leader       varchar(255) null comment '活动负责人',
    host_unit          varchar(255) not null comment '主办单位',
    host_unit_lever    varchar(255) not null comment '主办单位级别',
    event_address      varchar(255) null comment '活动地点',
    event_content      text         null comment '活动内容',
    event_start_time   date         not null comment '活动开始时间',
    event_end_time     date         not null comment '活动结束时间',
    attend_class       longtext     not null comment '参于班级',
    attend_class_num   int          null comment '参于班级数',
    attend_teacher     longtext     not null comment '参于教师',
    attend_teacher_num int          null comment '参于教师数',
    attend_student_num int          null comment '参于学生数'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='思政德育活动';

# 字典
# 主办单位级别 host_unit
# 值   名称   编号
# 1 国家级   host_unit_1
# 2 省级    host_unit_2
# 3 地市级   host_unit_3
# 9 其他     host_unit_9

# 字典
# 德育活动类型 moral_type
# 值   名称  编号
# 1 讲座报告  moral_type_1
# 2 征文演讲  moral_type_2
# 3 文艺表演  moral_type_3
# 4 参观走访  moral_type_4
# 5 志愿服务  moral_type_5
# 6 社会实践  moral_type_6
# 7 分享交流  moral_type_7
# 9 其他     moral_type_9

# 字典
# 德育活动主题 moral_theme
# 值   名称                            编号
# 1 习近平新时代中国特色社会主义思想教育     moral_theme_1
# 2 “四史”教育                         moral_theme_2
# 3 社会主义核心价值观教育                 moral_theme_3
# 4 爱国主义教育                          moral_theme_4
# 5 中华优秀传统文化教育                  moral_theme_5
# 6 法治教育                            moral_theme_6
# 7 国家安全教育                          moral_theme_7
# 8 劳动教育                            moral_theme_8
# 9 健康教育                            moral_theme_9
# 10 职业生涯教育                         moral_theme_10
# 99 其他                             moral_theme_99


-- 教师进修
DROP TABLE IF EXISTS `teacher_trainee`;
CREATE TABLE teacher_trainee
(
    id                   bigint       not null comment '主键编号'
        primary key,
    create_user_id       bigint       null comment '创建人',
    create_date          datetime     null comment '创建时间',
    modify_user_id       bigint       null comment '修改人',
    modify_date          datetime     null comment '修改时间',
    delete_mark          int          not null comment '删除标记',
    enabled_mark         int          not null comment '有效标志',
    sort_code            int          null comment '序号',

    teacher_id           bigint       not null comment '进修教师id',
    teacher_name         varchar(255) not null comment '教师姓名',
    teacher_user_name    varchar(255) not null comment '教师工号or教师oa',
    trainee_name         varchar(255) not null comment '进修培训名称',
    trainee_number       varchar(255) not null comment '进修培训编号',
    trainee_theme        varchar(255) null comment '进修培训主题',
    trainee_content      varchar(255) null comment '进修培训内容',
    trainee_main_speaker varchar(255) null comment '进修培训主讲人',
    start_time           date         not null comment '进修培训开始日期',
    end_time             date         not null comment '进修培训截止日期',
    trainee_place        varchar(255) not null comment '进修培训地点',
    out_date             date         null comment '出国（境）日期',
    out_purpose          varchar(255) null comment '出国（境）目的',
    out_unit             varchar(255) null comment '派出单位',
    approval_unit        varchar(255) null comment '审批单位',
    approval_date        date         null comment '审批日期',
    return_date          date         null comment '应回国日期'

) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='教师进修';
# 字典
# 进修学习地点   trainee_place
# 值   名称
# 1 国内进修培训  trainee_place_1
# 2 国外进修培训  trainee_place_2

# 字典
# 出国境目的  out_purpose
# 值   名称                            编号
# 01 常驻国外使馆、领事馆                 out_purpose_01
# 02 常驻国际组织及其代表机构               out_purpose_02
# 04 常驻国外其他组织                       out_purpose_04
# 05 短期派驻国外使馆、领事馆                   out_purpose_05
# 06 短期派驻国际组织及其他代表机构            out_purpose_06
# 08 短期派驻国外其他组织                     out_purpose_08
# 10 党政代表团出访                            out_purpose_10
# 11 军事代表团出访                            ......
# 12 经济贸易和财务方面代表团出访与洽谈
# 13 学术、文艺、体育代表团和其他社会团体出访
# 14 参加国际性的各类比赛
# 15 参加交易会和展览会
# 20 引进技术和设备
# 21 商务出国
# 22 实习
# 23 监造
# 30 援外技术工作
# 31 援建工作
# 32 援外培训工作
# 33 劳务出口
# 34 合营工程
# 36 航空、邮电、海运、公路等国际联运业务
# 37 随船工作
# 39 科技合作项目
# 40 考察
# 41 领奖
# 42 参加各种会议
# 43 进修
# 44 讲学
# 45 公派留学
# 46 自费留学
# 47 自费公派留学
# 60 旅游
# 61 探亲
# 62 会友
# 63 结婚
# 64 继承财产
# 65 就业
# 66 定居
# 70 特殊任务
# 80 换发护照
# 99 其他原因出国

-- 课程互动记录表
DROP TABLE IF EXISTS `classroom_interaction_record`;
CREATE TABLE `classroom_interaction_record`
(
    `id`                            bigint       NOT NULL COMMENT '主键编号',
    `create_user_id`                bigint       NULL DEFAULT NULL COMMENT '创建人',
    `create_date`                   datetime     NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id`                bigint       NULL DEFAULT NULL COMMENT '修改人',
    `modify_date`                   datetime     NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark`                   int          NOT NULL COMMENT '删除标记',
    `enabled_mark`                  int          NOT NULL COMMENT '有效标志',
    `sort_code`                     int          NULL COMMENT '序号',


    `class_grade_id`                varchar(255) NOT NULL COMMENT '班级id',
    `class_grade_name`              varchar(255) NOT NULL COMMENT '班级',
    `major_information_id`          varchar(255) NOT NULL COMMENT '专业信息id',
    `major_information`             varchar(255) NOT NULL COMMENT '专业信息',
    `grade_information_id`          varchar(255) NOT NULL COMMENT '年级信息id',
    `grade_information`             varchar(255) NOT NULL COMMENT '年级信息',
    `week_time`                     varchar(255) NOT NULL COMMENT '星期',
    `week_number`                   int          NOT NULL COMMENT '周次',
    `section_number`                varchar(255) NOT NULL COMMENT '节次',
    `course_schedule_number`        varchar(255) NOT NULL COMMENT '排课号',
    `course_name`                   varchar(255) NOT NULL COMMENT '课程名称',
    `course_nunber`                 varchar(255) NOT NULL COMMENT '课程号',
    `class_data`                    date         NOT NULL COMMENT '上课日期',
    `class_time`                    varchar(255) NOT NULL COMMENT '上课时间',
    `number_interactions_initiated` bigint       NOT NULL COMMENT '互动发起次数',
    `number_student_participations` bigint       NOT NULL COMMENT '学生参与总次数',
    `attendance_rate`               varchar(255) NOT NULL COMMENT '出勤率',
    PRIMARY KEY (`id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT = '课程互动记录';

-- 综合成绩
DROP TABLE IF EXISTS `combine_cert`;
CREATE TABLE `combine_cert`
(
    id                             bigint         not null comment '主键编号'
        primary key,
    create_user_id                 bigint         null comment '创建人',
    create_date                    datetime       null comment '创建时间',
    modify_user_id                 bigint         null comment '修改人',
    modify_date                    datetime       null comment '修改时间',
    delete_mark                    int            not null comment '删除标记',
    enabled_mark                   int            not null comment '有效标志',
    sort_code                      int            null comment '序号',

    base_semester                  varchar(256)   NOT NULL COMMENT '学期',
    base_semester_id               bigint         NOT NULL COMMENT '学期id',
    user_id                        bigint         NOT NULL COMMENT '学生id',
    student_id                     varchar(255)   NOT NULL COMMENT '学号',
    student_name                   varchar(255)   not null comment '学生姓名',
    student_gender                 varchar(32)    NOT NULL COMMENT '性别',
    base_grade_name                VARCHAR(256)   not null COMMENT '年级',
    class_id                       bigint         not null comment '班级id',
    class_name                     varchar(64)    not null comment '学生班级',
    base_major_name                VARCHAR(256)   not null COMMENT '专业名称',
    teacher_name                   varchar(64)    NOT NULL COMMENT '教师姓名',

    thought_politics_score         DECIMAL(10, 1) not null comment '思想政治成绩',
    culture_score                  DECIMAL(10, 1) not null comment '文化课成绩',
    expertise_score                DECIMAL(10, 1) not null comment '专业技能成绩',
    student_health_score           DECIMAL(10, 1) not null comment '学生体质健康成绩',
    omprehensive_training          DECIMAL(10, 1) not null comment '综合实训成绩',
    post_exercitation_score        DECIMAL(10, 1) not null comment '岗位实践成绩',
    comprehensive_evaluation_score DECIMAL(10, 1) not null comment '综合评价成绩'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    comment '综合成绩';


-- 教职工考核
DROP TABLE IF EXISTS `faculty_assess`;
CREATE TABLE `faculty_assess`
(
    id                       bigint       not null comment '主键编号'
        primary key,
    create_user_id           bigint       null comment '创建人',
    create_date              datetime     null comment '创建时间',
    modify_user_id           bigint       null comment '修改人',
    modify_date              datetime     null comment '修改时间',
    delete_mark              int          not null comment '删除标记',
    enabled_mark             int          not null comment '有效标志',
    sort_code                int          null comment '序号',

    teacher_name             varchar(255) NOT NULL COMMENT '考核教师',
    assess_date              date         NOT NULL COMMENT '考核日期',
    assess_type              varchar(255) COMMENT '考核类型',
    xjr_department_name      varchar(255) NOT NULL COMMENT '考核部门',
    dept_assess_result       varchar(255) NOT NULL COMMENT '部门考核结果',
    asses_head_assess_result varchar(255) COMMENT '考核负责人',
    school_assess_result     varchar(255) COMMENT '学校考核结果',
    assess_content           varchar(500) COMMENT '考核内容',
    teacher_user_name        varchar(64)  NOT NULL COMMENT '用户名'

) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    comment '教职工考核';

# 字典
# a_type  考核类型
# 值   名称
# 1   平时考核 a_type1
# 2   年度考核 a_type2
# 3   特定考核 a_type3
#
# a_result  考核结果
# 值   名称
# 1   优秀 a_result1
# 2   称职 a_result2
# 3   基本称职 a_result3
# 4   不称职 a_result4
# 5   不定等次 a_result5
# 6   不进行考核 a_result6

-- 实习报告表
DROP TABLE IF EXISTS `internship_report`;
CREATE TABLE `internship_report`
(
    `id`                        bigint       NOT NULL COMMENT '主键编号',
    `create_user_id`            bigint       NULL DEFAULT NULL COMMENT '创建人',
    `create_date`               datetime     NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id`            bigint       NULL DEFAULT NULL COMMENT '修改人',
    `modify_date`               datetime     NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark`               int          NOT NULL COMMENT '删除标记',
    `enabled_mark`              int          NOT NULL COMMENT '有效标志',
    `sort_code`                 int          NULL DEFAULT NULL COMMENT '序号',

    `student_name`              varchar(255) NOT NULL COMMENT '姓名',
    `student_id`                bigint       NOT NULL COMMENT '学号',
    `class_name`                varchar(255) NOT NULL COMMENT '班级',
    `teacher_id`                bigint       NOT NULL COMMENT '班主任id',
    `teacher_name`              varchar(255) NOT NULL COMMENT '班主任',
    `program_name`              varchar(255) NOT NULL COMMENT '实习计划名称',
    `internship_unit`           varchar(255) NULL COMMENT '实习单位',
    `internship_positions`      varchar(255) NULL COMMENT '实习岗位',
    `internship_report_content` text         NOT NULL COMMENT '实习报告内容',
    `report_category`           varchar(255) NOT NULL COMMENT '报告类型',
    `status`                    int          NOT NULL COMMENT '状态',
    PRIMARY KEY (`id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT = '实习报告表';


-- 教学质量与评价
DROP TABLE IF EXISTS `teaching_quality_evaluation`;
CREATE TABLE `teaching_quality_evaluation`
(
    `id`                bigint       NOT NULL COMMENT '主键编号',
    `create_user_id`    bigint       NULL DEFAULT NULL COMMENT '创建人',
    `create_date`       datetime     NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id`    bigint       NULL DEFAULT NULL COMMENT '修改人',
    `modify_date`       datetime     NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark`       int          NOT NULL COMMENT '删除标记',
    `enabled_mark`      int          NOT NULL COMMENT '有效标志',
    `sort_code`         int          NULL DEFAULT NULL COMMENT '序号',


    `grade`             varchar(255) NOT NULL COMMENT '年级',
    `class_name`        varchar(255) NOT NULL COMMENT '班级',
    `major_name`        varchar(255) NOT NULL COMMENT '专业名称',
    `course_name`       varchar(255) NOT NULL COMMENT '课程名称',
    `course_code`       varchar(255) NOT NULL COMMENT '课程编号',
    `course_type`       varchar(255) NOT NULL COMMENT '课程分类',
    `teacher_id`        bigint       NOT NULL COMMENT '任课教师id',
    `teacher_name`      varchar(255) NOT NULL COMMENT '任课教师',
    `course_evaluation` varchar(255) NOT NULL COMMENT '课程质量评价',
    `student_number`    int          NOT NULL COMMENT '参评学生数',
    `evaluation_date`   date         NOT NULL COMMENT '评价完成日期',

    PRIMARY KEY (`id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT = '教学质量与评价';

-- 教室视频流维护
DROP TABLE IF EXISTS `teacher_video_streaming_maintenance`;
CREATE TABLE `teacher_video_streaming_maintenance`
(
    id                     bigint       not null comment '主键编号'
        primary key,
    create_user_id         bigint       null comment '创建人',
    create_date            datetime     null comment '创建时间',
    modify_user_id         bigint       null comment '修改人',
    modify_date            datetime     null comment '修改时间',
    delete_mark            int          not null comment '删除标记',
    enabled_mark           int          not null comment '有效标志',
    sort_code              int          null comment '序号',

    `classroom_number`     varchar(255) NULL DEFAULT NULL COMMENT '教室号',
    `video_id`             varchar(255) NULL DEFAULT NULL COMMENT '视频设备id',
    `video_device_name`    varchar(255) NULL DEFAULT NULL COMMENT '视频设备点位名称',
    `video_stream_address` varchar(255) NULL DEFAULT NULL COMMENT '视频流地址'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='教室视频流维护';

-- 网络课程在线学习记录
DROP TABLE IF EXISTS `inter_class_studyrecode`;
CREATE TABLE `inter_class_studyrecode`
(
    id                     bigint       not null comment '主键编号'
        primary key,
    create_user_id         bigint       null comment '创建人',
    create_date            datetime     null comment '创建时间',
    modify_user_id         bigint       null comment '修改人',
    modify_date            datetime     null comment '修改时间',
    delete_mark            int          not null comment '删除标记',
    enabled_mark           int          not null comment '有效标志',
    sort_code              int          null comment '序号',

    inter_class_name       varchar(255) not null comment '网络课程名称',
    inter_class_source     varchar(255) not null comment '网络课程来源',
    inter_class_plaform    varchar(255) not null comment '网络课程所属平台名称',
    teacher_id             bigint       not null comment '主讲教师id',
    teacher_oa             varchar(255) not null comment '主讲教师工号',
    teacher_name           varchar(255) not null comment '主讲教师姓名',
    student_study_data     date         not null comment '学生学习日期',
    student_study_timesize double       not null comment '学生学习时长',
    student_study_number   int          not null comment '学生学习人次',
    student_submit_number  int          not null comment '学生提交作业数',
    teach_number           int          not null comment '教学资源数',
    answer_number          int          not null comment '答疑讨论数',
    input_people_id        bigint       null comment '录入人id',
    input_people           varchar(255) null comment '录入人',
    input_data             date         not null comment '录入日期'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 comment '网络课程在线学习记录';


-- 计划课程管理表

DROP TABLE IF EXISTS plan_course_manage;
CREATE TABLE plan_course_manage
(
    id                          bigint       not null comment '主键编号' primary key,
    create_user_id              bigint       null comment '创建人',
    create_date                 datetime     null comment '创建时间',
    modify_user_id              bigint       null comment '修改人',
    modify_date                 datetime     null comment '修改时间',
    delete_mark                 int          not null comment '删除标记',
    enabled_mark                int          not null comment '有效标志',
    sort_code                   int          null comment '序号',


    base_semester_id            bigint       not null comment '学期主键id(base_semester)',
    base_semester               varchar(256) not null comment '学期',

    course_plan_code            varchar(256) not null comment '计划课程编号',

    base_course_subject_id      bigint       not null comment '课程id(base_course_subject)',
    base_course_subject_name    varchar(256) not null comment '课程学科名称(base_course_subject)',
    base_course_subject_code    varchar(256) not null comment '课程学科代码(base_course_subject)',

    core_course                 varchar(32)  not null comment '是否专业核心课程(base_course_subject)',
    textbook_publication_number longtext     not null comment '教材出版号',

    teaching_method             varchar(256) not null comment '主要授课方式',
    exam_method                 varchar(32)  not null comment '考试方式',

    teacher_name                varchar(256) not null comment '主讲教师',
    teacher_id                  bigint       not null comment '主讲教师id',
    teacher_user_name           varchar(256) not null comment '主讲教师工号',

    course_certificate          varchar(32)  not null comment '是否课证融通课程',
    cert_id                     longtext     null comment '证书id',
    contest_name                longtext     null comment '相关竞赛名称'

) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='计划课程管理表';

-- 数据字典

-- 主要授课方式 teaching_method
-- 值         名称                    编号
-- 1  讲授 teaching_method_1
-- 2  讲授+演示 teaching_method_2
-- 3  实践操作指导 teaching_method_3

-- 学校成果管理
DROP TABLE IF EXISTS `base_course_subject`;
CREATE TABLE base_school_outcome
(
    id              bigint       not null comment '主键编号' primary key,
    create_user_id  bigint       null comment '创建人',
    create_date     datetime     null comment '创建时间',
    modify_user_id  bigint       null comment '修改人',
    modify_date     datetime     null comment '修改时间',
    delete_mark     int          not null comment '删除标记',
    enabled_mark    int          not null comment '有效标志',
    sort_code       int          null comment '序号',
-- 共有属性
    outcome_name    VARCHAR(256) not null comment '成果名称',
    outcome_type    VARCHAR(16)  not null comment '成果类型',
    completion_unit VARCHAR(256) not null comment '完成单位',
    completion_date date         not null comment '完成时间',
    completer       VARCHAR(256) not null comment '完成人',
    overview        LONGTEXT     null comment '概述',
    file_id         LONGTEXT     not null comment '辅佐材料',
--     论文
    media_name      VARCHAR(256) null comment '媒体名称',
    media_type      VARCHAR(16)  null comment '媒体类型',
    issue_num_cn    VARCHAR(256) null comment '国内统一刊号CN',
    issue_num_intl  VARCHAR(256) null comment '国际标准刊号',
    journal_level   VARCHAR(32)  null comment '期刊级别',
    issued_date     date         null comment '发表时间',
--     专著
    monograph_type  VARCHAR(16)  null comment '专著类型',
    book_num_intl   VARCHAR(256) null comment '国际标准书号',
    publish_house   VARCHAR(256) null comment '出版社',
    publish_date    date         null comment '出版日期',
    edition         VARCHAR(128) null comment '版次',
--     专利
    patent_type     VARCHAR(64)  null comment '专利类型',
    patent_num      VARCHAR(256) null comment '专利号',
    patent_req      date         null comment '专利申请日期',
    patent_authz    date         null comment '专利授权日期',
    patentee        VARCHAR(256) null comment '专利权人',
--     软件著作权
    dev_finish_date date         null comment '开发完成日期',
    version_num     VARCHAR(64)  null comment '版本号',
    dev_status      VARCHAR(64)  null comment '开发情况',
--     教学成果奖
    teach_outcome   VARCHAR(64)  null comment '教学成果类型',
    innovation      LONGTEXT     null comment '创新点',
--     其它
    outcome_desc    LONGTEXT     null comment '成果说明'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='学校成果管理';


-- 实习计划管理表

DROP TABLE IF EXISTS internship_plan_manage;
CREATE TABLE internship_plan_manage
(
    id                                bigint       not null comment '主键编号' primary key,
    create_user_id                    bigint       null comment '创建人',
    create_date                       datetime     null comment '创建时间',
    modify_user_id                    bigint       null comment '修改人',
    modify_date                       datetime     null comment '修改时间',
    delete_mark                       int          not null comment '删除标记',
    enabled_mark                      int          not null comment '有效标志',
    sort_code                         int          null comment '序号',


    base_semester_id                  bigint       not null comment '学期主键id(base_semester)',
    base_semester                     varchar(256) not null comment '学期',

    internship_plan_name              varchar(256) not null comment '实习计划名称',
    internship_plan_type              varchar(256) not null comment '实习计划类型',

    training_base_id                  bigint       not null comment '实训基地主键id(school_training_base and outside_training_base)',
    training_base_name                varchar(256) not null comment '实训基地名称',

    professional_relevance_student    varchar(256) null comment '专业对口程度',

    internship_whereabouts            varchar(256) null comment '实习去向',

    insurance_purchaser               varchar(256) null comment '保险购买方',
    insurance_type                    varchar(256) null comment '购买保险种类',

    start_time                        date         not null comment '实习开始时间',
    end_time                          date         not null comment '实习结束时间',

    enterprise_industry               varchar(256) null comment '企业所属行业',
    internship_unit_name              varchar(256) not null comment '实习单位名称',
    internship_unit_address           varchar(256) null comment '实习单位地址',
    internship_unit_source            varchar(256) not null comment '实习单位来源',
    internship_job_name               varchar(256) not null comment '实习岗位名称',

    professional_relevance_enterprise varchar(256) null comment '专业对口程度',

    lodging_plan                      varchar(256) null comment '住宿安排'

) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='实习计划管理表';

-- 实习计划参与人表
DROP TABLE IF EXISTS internship_plan_manage_participant;
CREATE TABLE internship_plan_manage_participant
(
    id                        bigint       not null comment '主键编号' primary key,
    create_user_id            bigint       null comment '创建人',
    create_date               datetime     null comment '创建时间',
    modify_user_id            bigint       null comment '修改人',
    modify_date               datetime     null comment '修改时间',
    delete_mark               int          not null comment '删除标记',
    enabled_mark              int          not null comment '有效标志',
    sort_code                 int          null comment '序号',

    internship_plan_manage_id bigint       not null comment '实习计划id',

    participant_id            bigint       null comment '实习参与人id',
    participant_user_id       varchar(256) null comment '实习参与人学号',
    participant_name          varchar(256) null comment '实习参与人姓名',

    base_major_name           varchar(255) not null comment '所属专业名称',
    base_major_id             bigint       not null comment '所属专业id(base_major)',

    class_name                varchar(256) not null comment '班级',
    class_id                  bigint       not null comment '班级id(base_class)'

) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='实习计划参与人表';


-- 数据字典


-- 实习类别  participant_type
-- 值  名称      编号
-- 1 认识实习  p_type_1
-- 2 岗位实习 p_type_2
-- 9 其他         p_type_9


-- 专业对口程度类型  pr_ent_type
-- 值  名称      编号
-- 1 不对口 pr_ent_type_1
-- 2 基本对口 pr_ent_type_2
-- 3 非常对口 pr_ent_type_3


-- 实习去向类型  int_where_type
-- 值  名称                          编号
-- 1    本市                      iw_type_1
-- 2    省内（非本市）iw_type_2
-- 3     省外                    iw_type_3
-- 4     国外                    iw_type_4

-- 保险购买方类型 ins_p_type
-- 值  名称      编号
-- 1  学校 ins_p_type_1
-- 2  企业 ins_p_type_2
-- 9 其他   ins_p_type_9

-- 购买保险种类 insurance_type
-- 值  名称      编号
-- 1  学生实习责任保险  ins_type_1
-- 2  意外险  ins_type_2
-- 9  其他险  ins_type_9

-- 所属行业 SECTOR（已有字典,但是字典值不同）
-- 值         名称                                 编号
-- A        农、林、牧、渔业        sector_a
-- B        采矿业                            sector_b
-- C        制造业                            sector_c
-- D        电力、热力、燃气及水生产和供应业    sector_d
-- E        建筑业                            sector_e
-- F        批发和零售业                        sector_f
-- G        交通运输、仓储和邮政业                sector_g
-- H        住宿和餐饮业                        sector_h
-- I        信息传输、软件和信息技术服务业        sector_i
-- J        金融业                            sector_j
-- K        房地产业                            sector_k
-- L        租赁和商务服务业                    sector_l
-- M        科学研究和技术服务业                sector_m
-- N        水利、环境和公共设施管理业        sector_n
-- O        居民服务、修理和其他服务业        sector_o
-- P        教育                            sector_p
-- Q        卫生和社会工作                        sector_q
-- R        文化、体育和娱乐业                    sector_r
-- S        公共管理、社会保障和社会组织        sector_s
-- T        国际组织                            sector_t

-- 实习单位来源 unit_source
-- 值         名称                                 编号
-- 1  学校合作企业                            unit_source_1
-- 2  他人介绍（非学校合作企业） unit_source_2
-- 3 自己寻找                                      unit_source_3
-- 9 其他                                             unit_source_9

-- 住宿安排 lodging_plan
-- 值         名称                    编号
-- 0  不安排住宿         lodging_plan_0
-- 1  安排住宿             lodging_plan _1


-- 课程学科管理
DROP TABLE IF EXISTS `base_course_subject`;
CREATE TABLE base_course_subject
(
    id               bigint       not null comment '主键编号' primary key,
    create_user_id   bigint       null comment '创建人',
    create_date      datetime     null comment '创建时间',
    modify_user_id   bigint       null comment '修改人',
    modify_date      datetime     null comment '修改时间',
    delete_mark      int          not null comment '删除标记',
    enabled_mark     int          not null comment '有效标志',
    sort_code        int          null comment '序号',

    name             VARCHAR(64)  not null comment '课程学科名称',
    code             VARCHAR(16)  not null comment '课程学科代码',
    course_category  VARCHAR(16)  not null comment '课程类别',
    course_nature    VARCHAR(16)  not null comment '课程性质',
    course_attribute VARCHAR(16)  not null comment '课程属性',
    course_type      VARCHAR(16)  not null comment '课程分类',
    subject_quality  VARCHAR(16)  not null comment '学科类别',
    core_course      VARCHAR(16)  not null comment '是否专业核心课程',
    theory_time      INT          not null comment '理论教学时数',
    practice_time    INT          not null comment '实践教学时数',
    online_course    VARCHAR(1)   not null comment '是否有线上课程',
    course_url       LONGTEXT     null comment '线上课程网址',
    ideology_demo    VARCHAR(1)   not null comment '是否思政示范课程',
    course_resource  VARCHAR(10)  null comment '课堂教学设计主要选用课程资源',
    emulation_time   INT          not null comment '虚拟仿真实训课时数',
    emulation_name   VARCHAR(256) null comment '主要开设虚拟仿真实训项目名称',
    experiment_name  VARCHAR(256) null comment '主要开设实验项目名称',
    training_name    VARCHAR(256) null comment '主要开设实训项目名称',
    internship_name  VARCHAR(256) null comment '主要开设实习项目名称'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='课程学科管理';

-- 数据字典项               		编号                          	   备注
-- 课程类别代码               course_category			          	课程类别代码
-- 字典名        编号               字典值
-- 理论类     course_cat_1            1
-- 语言类     course_cat_2            2
-- 实验实训类  course_cat_3            3
-- 体育类     course_cat_4            4
-- 实践类     course_cat_5            5
-- 艺术类     course_cat_6            6
-- 其他类     course_cat_9            9

-- 数据字典项               		编号                          	   备注
-- 课程性质代码               course_nature			          	课程性质代码
-- 字典名        编号               字典值
-- 公共基础课   course_nature_1       1
-- 专业基础课   course_nature_2       2
-- 专业核心课   course_nature_3       3
-- 专业拓展课   course_nature_4       4
-- 其他        course_nature_9       9

-- 数据字典项               		编号                          	   备注
-- 课程属性代码               course_attribute			          	课程性质代码
-- 字典名        编号               字典值
-- 必修课      c_attribute_1       1
-- 公共选修课   c_attribute_2       2
-- 专业选修课   c_attribute_3       3
-- 其他        c_attribute_9       9

-- 数据字典项               		编号                          	   备注
-- 课程分类代码                course_type			          	课程分类代码
-- 字典名              编号               字典值
-- A类(纯理论课)      course_type_1        1
-- B类(理论＋实践)课   course_type_2        2
-- C类(纯实践课)      course_type_3        3

-- 数据字典项               		编号                          	   备注
-- 学科类别代码               subject_quality			          	学科类别代码
-- 字典名             编号               字典值
-- 哲学         s_quality_01       01
-- 经济学       s_quality_02       02
-- 法学         s_quality_03       03
-- 教育学       s_quality_04       04
-- 文学         s_quality_05       05
-- 历史学       s_quality_06       06
-- 理学         s_quality_07       07
-- 工学         s_quality_08       08
-- 农学         s_quality_09       09
-- 医学         s_quality_10       10
-- 军事学       s_quality_11       11
-- 管理学       s_quality_12       12
-- 艺术学       s_quality_13       13

-- 数据字典项               		编号                          	   备注
-- 主要选用课程资源代码               course_resource			  主要选用课程资源代码
-- 字典名             编号               字典值
-- 媒体素材         c_resource_1       1
-- 试题            c_resource_2       2
-- 试卷            c_resource_3       3
-- 课件            c_resource_4       4
-- 案例            c_resource_5       5
-- 文献资料         c_resource_6       6
-- 网络课程         c_resource_7       7
-- 教学工具软件      c_resource_8       8


